@@ -138,7 +138,7 @@ pub fn expand_node(
         // Just special casing for efficiency
         // We can't use unwrap because we still want to propagate errors from
         // stuff like dtype mismatches or constraint violations.
-        return circuit.map_children_idxs(|i| Ok(inputs[i].clone()));
+        return circuit.replace_children(inputs.clone());
     }
 
     let batch_ranks: Vec<usize> = zip(circuit.children(), inputs)
@@ -1023,6 +1023,14 @@ where
     ReplaceExpander::new_fn(replacer).replace_expand(circuit)
 }
 
+pub fn replace_expand_map(
+    circuit: CircuitRc,
+    map: Vec<(CircuitRc, CircuitRc)>,
+) -> Result<CircuitRc> {
+    ReplaceExpander::new_noop()
+        .replace_expand_with_map(circuit, &ReplaceMapRc::new(map.into_iter().collect()))
+}
+
 #[test]
 fn check_map_override() {
     let m: BTreeMap<_, _> = [(1, 2), (3, 4), (5, 7), (3, 9)].into_iter().collect();
