@@ -19,7 +19,7 @@ use rr_util::{
     rearrange_spec::RearrangeSpec,
     symbolic_size::SymbolicSizeProduct,
     tensor_util::{right_align_shapes, Shape},
-    util::{arc_unwrap_or_clone, counts_g_1, is_unique},
+    util::{arc_unwrap_or_clone, counts_g_1, is_unique, HashBytes},
     IndexSet,
 };
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet, FxHasher};
@@ -27,17 +27,20 @@ use thiserror::Error;
 use uuid::{uuid, Uuid};
 
 use crate::{
+    auto_name::OperatorPriority,
+    circuit_info::CachedCircuitInfo,
     circuit_node_auto_impl, circuit_node_extra_impl,
     circuit_node_private::{CircuitNodeHashItems, CircuitNodeSetNonHashInfo},
-    circuit_utils::OperatorPriority,
-    deep_map, deep_map_op, deep_map_op_context_preorder_stoppable, deep_map_pre_new_children,
-    deep_map_preorder_unwrap,
-    expand_node::{ExpandError, ReplaceExpander, ReplaceMapRc},
-    named_axes::{deep_strip_axis_names, propagate_named_axes},
+    circuit_utils::{
+        deep_map, deep_map_op, deep_map_op_context_preorder_stoppable, deep_map_pre_new_children,
+        deep_map_preorder_unwrap, visit_circuit_unwrap,
+    },
+    constant::Symbol,
+    expand_node::{replace_expand_map, ExpandError},
+    named_axes::{deep_strip_axis_names, get_axis_leaves, propagate_named_axes},
     new_rc_unwrap,
     prelude::*,
-    visit_circuit_unwrap, CachedCircuitInfo, HashBytes, PyCircuitBase, Rearrange, Symbol,
-    TensorEvalError,
+    PyCircuitBase, Rearrange, TensorEvalError,
 };
 
 /// can also be thought of as lambda from lambda calculus (expression with free variables + list of these variables)
@@ -214,18 +217,15 @@ impl ModuleSpec {
             .context("batch shapes didn't match for module")?;
 
         // TODO: fix this being uncached!
-        let out = ReplaceExpander::new_noop()
-            .replace_expand_with_map(
-                self.circuit.clone(),
-                &ReplaceMapRc::new(
-                    self.arg_specs
-                        .iter()
-                        .zip(nodes)
-                        .map(|(arg_spec, node)| (arg_spec.symbol.crc(), node.clone()))
-                        .collect(),
-                ),
-            )
-            .context("replace expand failed from substitute")?;
+        let out = replace_expand_map(
+            self.circuit.clone(),
+            self.arg_specs
+                .iter()
+                .zip(nodes)
+                .map(|(arg_spec, node)| (arg_spec.symbol.crc(), node.clone()))
+                .collect(),
+        )
+        .context("replace expand failed from substitute")?;
 
         assert!(out.ndim() >= self.circuit.ndim());
         let out_batch_rank = out.ndim() - self.circuit.ndim();
@@ -448,6 +448,13 @@ impl ModuleSpec {
         Ok(out)
     }
 
+    pub fn is_batchable_to_inputs(&self, dim: usize) -> Option<Vec<usize>> {
+        let (_, _, leavs_axis, leaves_non_axis) =
+            get_axis_leaves(self.circuit.clone(), dim).ok()?;
+        dbg!((&dim, &leavs_axis, &leaves_non_axis));
+        None
+    }
+
     pub fn compute_non_children_hash(&self, hasher: &mut blake3::Hasher) {
         for arg_spec in &self.arg_specs {
             // this is fine because each item is fixed size and we delimit with node hashs (which are uu)
@@ -827,7 +834,7 @@ impl ModuleArgSpec {
         expandable = Self::default().expandable,
         ban_non_symbolic_size_expand = Self::default().ban_non_symbolic_size_expand
     ))]
-    fn new(
+    pub fn new(
         symbol: Symbol,
         batchable: bool,
         expandable: bool,
@@ -997,7 +1004,7 @@ impl Module {
         let spec_resized = self
             .spec
             .resize(self.args().map(|x| x.info().shape.clone()).collect())?;
-        let spec_circuit_resized = spec_resized.circuit.clone();
+        let spec_circuit_resized = substitute_all_modules(spec_resized.circuit.clone());
         assert!(spec_circuit_resized.info().shape[..] == self.info().shape[..]);
         let stripped = deep_strip_axis_names(spec_circuit_resized, &None);
         let out_named = propagate_named_axes(
@@ -1007,19 +1014,56 @@ impl Module {
                 .collect(),
             true,
         );
+
+        let mut seen: Vec<bool> = vec![false; self.num_children() - 1];
+        let mut axes_dead: Vec<bool> = vec![false; self.rank()];
         let mut result: Vec<Vec<Option<usize>>> =
             self.args().map(|c| vec![None; c.info().rank()]).collect();
+
         visit_circuit_unwrap(out_named, |x| {
             if let Some(sym) = x.as_symbol() {
                 if let Some(i) = self.spec.arg_specs.iter().position(|x| {
                     x.symbol.uuid == sym.uuid && x.symbol.info().name == sym.info().name
                 }) {
+                    let mut vec = vec![None; sym.rank()];
                     for (k, v) in &sym.info().named_axes {
-                        result[i][*k as usize] = Some(v.parse::<usize>().unwrap());
+                        vec[*k as usize] = Some(v.parse::<usize>().unwrap());
+                    }
+                    if !seen[i] {
+                        result[i] = vec;
+                        seen[i] = true;
+                    } else {
+                        for (k, v) in vec.iter().enumerate() {
+                            if result[i][k] != *v {
+                                result[i][k] = None;
+                                (*v).or(result[i][k]).map(|ax| {
+                                    axes_dead[ax] = true;
+                                });
+                            }
+                        }
+                    }
+                }
+            } else {
+                let mut children_named_axes: HashSet<Name> = HashSet::default();
+                for m in x.children() {
+                    children_named_axes.extend(m.info().named_axes.values())
+                }
+                for (_, v) in &x.info().named_axes {
+                    if !children_named_axes.contains(&v) {
+                        axes_dead[v.parse::<usize>().unwrap()] = true;
                     }
                 }
             }
         });
+        for m in &mut result {
+            for x in m.iter_mut() {
+                if let Some(y) = x {
+                    if axes_dead[*y] {
+                        *x = None;
+                    }
+                }
+            }
+        }
         Ok(result)
     }
 }
@@ -1217,7 +1261,7 @@ impl Module {
         )
     }
 
-    fn child_axis_map_inputs(&self) -> Result<Vec<Vec<Option<usize>>>> {
+    pub fn child_axis_map_inputs(&self) -> Result<Vec<Vec<Option<usize>>>> {
         let key = (
             self.spec.get_hashable(),
             self.args().map(|x| x.info().shape.clone()).collect(),
@@ -1228,6 +1272,7 @@ impl Module {
             return Ok(result);
         }
 
+        // could also exclude batch dim from shape & re-add them here
         let result = self.child_axis_map_inputs_uncached()?;
         MODULE_EXPANSIONS_AXIS_MAPS.with(|cache| cache.borrow_mut().insert(key, result.clone()));
         Ok(result)
@@ -1327,7 +1372,7 @@ impl AllModuleSubstituter {
             .substitute_with_mod_name(&nodes, None, None, m.info().name)
             .expect("module constructor should ensure this works")
         } else {
-            circuit.map_children_unwrap_idxs(|i| new_children[i].clone())
+            circuit.replace_children(new_children).unwrap()
         }
     }
 }
@@ -1363,7 +1408,7 @@ pub fn inline_single_callsite_modules(circuit: CircuitRc) -> CircuitRc {
                 .map(|i| *i == 1)
                 .unwrap_or(false)
             {
-                mn.map_children_idxs(|z| Ok(children[z].clone()))
+                mn.replace_children(children.clone())
                     .unwrap()
                     .substitute(None, None)
             } else {
