@@ -240,8 +240,13 @@ impl ToPyObject for EndDepthOpaqueIterativeMatcher {
             let dict = PyDict::new(py);
             dict.set_item("end_depth", self.end_depth).unwrap();
 
-            OpaqueIterativeMatcherVal::noop_traversal_raw()
-                .call_method(py, "filter", (), Some(dict))
+            SELF_MODULE
+                .call_method(
+                    py,
+                    "restrict",
+                    (OpaqueIterativeMatcherVal::noop_traversal_raw(),),
+                    Some(dict),
+                )
                 .unwrap()
         })
     }
@@ -312,7 +317,7 @@ macro_rules! defer_to_py {
 
 impl OpaqueIterativeMatcherVal {
     pub fn noop_traversal_raw() -> PyObject {
-        atr!(SELF_MODULE, traversal, raw)
+        pycall!(atr!(SELF_MODULE, new_traversal, raw), (), raw)
     }
 
     pub fn for_end_depth(end_depth: usize) -> Self {
@@ -343,6 +348,7 @@ impl OpaqueIterativeMatcherVal {
             cache_transform: bool,
             cache_update: bool,
             fancy_validate: bool,
+            assert_any_found: bool,
         ) -> PyResult<CircuitRc>;
 
         fn get(&self, circuit: CircuitRc, fancy_validate: bool) -> PyResult<BTreeSet<CircuitRc>>;
@@ -379,8 +385,13 @@ pub fn get_opaque_type_matcher(
             .call_method0(py, "new_not")
             .unwrap();
         kwargs.set_item("term_early_at", not_type_matcher).unwrap();
-        OpaqueIterativeMatcherVal::op_to_object(&traversal)
-            .call_method(py, "filter", (), Some(kwargs))
+        SELF_MODULE
+            .call_method(
+                py,
+                "restrict",
+                (OpaqueIterativeMatcherVal::op_to_object(&traversal),),
+                Some(kwargs),
+            )
             .unwrap()
             .extract(py)
             .unwrap()
