@@ -887,7 +887,7 @@ impl PrintOptions {
             let new_traversal_per_op = if !term_on_repeat_or_reference {
                 let new_traversal_per_op = op_traversal_per(traversal, circ.crc())
                     .context("traversal failed in printer")?;
-                if new_traversal_per_op.is_none() && circ.children().count() != 0 {
+                if new_traversal_per_op.is_none() && circ.num_children() != 0 {
                     // all finished case
                     // traversal can't be used with bijection, so we can add ... without having to parse later
                     result.push_str(" ...");
@@ -949,7 +949,7 @@ impl PrintOptions {
                     recurse(
                         child,
                         depth + 1,
-                        circ.children().count() == 1,
+                        circ.num_children() == 1,
                         result,
                         seen_hashes,
                         runnning_serial_number,
@@ -964,7 +964,7 @@ impl PrintOptions {
                 }
                 return Ok(());
             }
-            if selfy.only_child_below && circ.children().count() == 1 {
+            if selfy.only_child_below && circ.num_children() == 1 {
                 write!(
                     result,
                     "{}▼\n",
@@ -974,7 +974,7 @@ impl PrintOptions {
                 recurse(
                     &circ.children().next().unwrap(),
                     depth,
-                    circ.children().count() == 1,
+                    circ.num_children() == 1,
                     result,
                     seen_hashes,
                     runnning_serial_number,
@@ -1002,7 +1002,7 @@ impl PrintOptions {
                 recurse(
                     &child,
                     depth + 1,
-                    circ.children().count() == 1,
+                    circ.num_children() == 1,
                     result,
                     seen_hashes,
                     runnning_serial_number,
@@ -1024,7 +1024,7 @@ impl PrintOptions {
             recurse(
                 &**circuit,
                 0,
-                circuit.children().count() == 1,
+                circuit.num_children() == 1,
                 &mut result,
                 &mut seen_hashes,
                 &mut runnning_serial_number,
