@@ -6,7 +6,7 @@ use crate::circuit_optimizer::OptimizationContext;
 
 /// takes circuitrc bc convenient
 pub fn numel_sort_key(node: &CircuitRc) -> Vec<u8> {
-    (usize::MAX - node.info().numel().to_u64_digits()[0] as usize)
+    (usize::MAX - *node.info().numel().to_u64_digits().get(0).unwrap_or(&0) as usize)
         .to_be_bytes()
         .iter()
         .copied()
@@ -39,12 +39,12 @@ pub fn canonicalize_node_op(circuit: CircuitRc) -> Option<CircuitRc> {
         )),
         Circuit::Add(add) => {
             let mut nodes_sorted = add.children_sl().to_vec();
-            nodes_sorted.sort_by_key(numel_sort_key);
+            nodes_sorted.sort_by_cached_key(numel_sort_key);
             Some(Add::nrc(nodes_sorted, add.info().name))
         }
         Circuit::Einsum(einsum) => {
             let mut args_sorted = einsum.args_cloned();
-            args_sorted.sort_by_key(|(node, _ints)| numel_sort_key(node));
+            args_sorted.sort_by_cached_key(|(node, _ints)| numel_sort_key(node));
             Some(
                 Einsum::try_new(args_sorted, einsum.out_axes.clone(), einsum.info().name)
                     .unwrap()
