@@ -47,8 +47,9 @@ pub fn pull_concat_once(circuit: CircuitRc) -> Option<CircuitRc> {
             .and_then(concat_elim_identity)
             .unwrap_or(z.clone())
     };
-    let g =
-        &|x: Index| push_down_index_raw(&x, false, &mut |_, ix| Ok(f(ix)), None).unwrap_or(x.rc());
+    let g = &|x: Index| {
+        push_down_index_raw(&x, false, &mut |_, ix| Ok(f(ix)), None, false).unwrap_or(x.rc())
+    };
     let name_maybe = circuit
         .children()
         .filter(|x| x.is_concat())
@@ -315,6 +316,7 @@ pub fn index_concat_drop_unreached(index: &Index) -> Option<CircuitRc> {
     if concat
         .children()
         .any(|x| x.info().numel() == BigUint::from(0usize))
+        || concat.num_children() == 0
     {
         return None;
     }
