@@ -28,7 +28,7 @@ use rr_util::{
     pycall, python_error_exception, sv,
     tensor_util::{Shape, TorchDevice},
     timed, unwrap,
-    util::{arc_ref_clone, with_context_failable, HashBytes},
+    util::{arc_ref_clone, hash_to_hex, with_context_failable, HashBytes},
 };
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 use thiserror::Error;
@@ -187,9 +187,10 @@ impl Schedule {
                 if allow_missing {
                     continue;
                 }
-                return Err(PyErr::new::<pyo3::exceptions::PyKeyError, _>(
-                    "key circuit wasn't present in original",
-                )
+                return Err(PyErr::new::<pyo3::exceptions::PyKeyError, _>(format!(
+                    "key circuit {k_hex} wasn't present in original",
+                    k_hex = hash_to_hex(&k)
+                ))
                 .into());
             }
             // should be present because we checked above
@@ -198,7 +199,12 @@ impl Schedule {
                 Entry::Occupied
             );
             if entry.get().shape() != v.shape() {
-                bail!("tensor shape doesn't match entry shape!");
+                bail!(
+                    "key {k_hex}: original shape {orig_shape:?} doesn't match replacement shape {new_shape:?}",
+                    k_hex=hash_to_hex(&k),
+                    orig_shape = entry.get().shape(),
+                    new_shape = v.shape(),
+                );
             }
 
             *entry.into_mut() = ScheduleConstant::Tensor(v);
