@@ -50,18 +50,14 @@ pub fn distribute(
                 })
             };
         }
-        let updated = traversal
-            .match_iterate(einsum.children_sl()[operand_idx].clone())?
-            .unwrap_or_same(traversal)
-            .0;
 
-        if updated.all_finished() {
+        let (_, finished, traversal_per_child) =
+            traversal.match_iterate_continue(einsum.children_sl()[operand_idx].clone())?;
+
+        if finished {
             return Ok(einsum.rc());
         }
 
-        let traversal_per_child =
-            updated.per_child_with_term(einsum.children_sl()[operand_idx].num_children());
-
         let distributed = distribute_once_raw(
             &einsum,
             operand_idx,
