@@ -1,6 +1,6 @@
 use std::{
     collections::BTreeMap,
-    fmt::{Debug, Display},
+    fmt::{Debug, Display, Write},
     hash::{BuildHasher, Hash},
     mem,
     rc::Rc,
@@ -22,6 +22,14 @@ pub type HashBytes = [u8; 32];
 pub type EinsumAxes = TinyVecU8;
 pub type NamedAxes = BTreeMap<AxisInt, Name>;
 
+pub fn hash_to_hex(h: &HashBytes) -> String {
+    let mut s = String::new();
+    for &byte in h {
+        write!(&mut s, "{:02x}", byte).unwrap();
+    }
+    s
+}
+
 #[derive(FromPyObject, Copy, Clone, Debug, Eq, PartialEq, Ord, PartialOrd)]
 pub struct HashBytesToPy(pub HashBytes);
 
