@@ -4,6 +4,7 @@ import functools
 import hashlib
 import itertools
 import math
+import uuid
 from collections import Counter
 from typing import Callable, Dict, Iterable, List, Optional, Sequence, Set, Tuple, TypeVar, Union, cast, overload
 
@@ -23,10 +24,14 @@ from ._rust import (
     IterativeMatcherIn,
     Matcher,
     MatcherIn,
+    ModuleArgSpec,
+    PrintOptions,
     PushDownIndexEinsumNoopError,
     Rearrange,
     RearrangeSpec,
     Scalar,
+    SetSymbolicShape,
+    Symbol,
     TorchAxisIndex,
     add_elim_zeros,
     add_flatten_once,
@@ -40,17 +45,22 @@ from ._rust import (
 )
 from ._rust import extract_add as extract_add_rust
 from ._rust import (
+    get_free_symbols,
+    hash_tensor,
     index_elim_identity,
     make_broadcast,
+    module_remove_unused_inputs,
     new_traversal,
     permute_of_einsum_merge,
     pull_concat_once_raw,
+    pull_out_of_modules,
+    push_down_index,
     push_down_index_once,
     rearrange_elim_identity,
     rearrange_fuse,
     traverse_until_depth,
 )
-from .py_utils import I
+from .py_utils import I, make_index_at
 
 T = TypeVar("T")
 
@@ -661,22 +671,20 @@ def pull_through_add_concat(c: Add) -> Circuit:
 try_pull_through_add_concat = try_transform_wrapper(pull_through_add_concat)
 
 
-def hash_index(index: List[TorchAxisIndex]) -> bytes:
-    """Like index hash, but does not support hash_tensor_idx_by_value"""
-    index_hash = hashlib.blake2b(b"<INDEX>")
-    from interp.circuit.eq_by_big_hash import hash_add, hash_add_by_id
+def index_key(index: List[TorchAxisIndex]) -> tuple:
+    """Convert index attr into a hashable tuple"""
+    out = []
 
     for i in index:
         if isinstance(i, torch.Tensor):
-            hash_add(index_hash, i.shape)
-            hash_add_by_id(index_hash, i)
-        elif isinstance(i, (int, slice)):
-            hash_add(index_hash, i)
+            out.append(hash_tensor(i))
+        elif isinstance(i, slice):
+            out.append((i.start, i.stop, i.step))
+        elif isinstance(i, int):
+            out.append(i)
         else:
             assert_never(i)
-        index_hash.update(b"<ITEM>")
-    index_hash.update(b"<END>")
-    return index_hash.digest()
+    return tuple(out)
 
 
 def pull_through_add_index(c: Add) -> Index:
@@ -695,8 +703,8 @@ def pull_through_add_index(c: Add) -> Index:
             raise ImpossibleRewriteError(f"Child {k} is not an Index")
         items.append(k)
 
-    shapes_index_hash = {(k.node.shape, hash_index(k.idx)) for k in items}
-    if len(shapes_index_hash) != 1:
+    shapes_index = {(k.node.shape, index_key(k.idx)) for k in items}
+    if len(shapes_index) != 1:
         raise ImpossibleRewriteError
 
     return Index(
@@ -1356,9 +1364,10 @@ def split_reduce_concat_impl(
             assert (~to_set[idx]).any() or to_set[idx].nelement() == 0
             to_set[idx] = True
             if is_concat:
-                assert (torch.sort(to_set, descending=True)[0] == to_set).all()
+                # torch.sort on CUDA doesn't support bool
+                to_set_long = to_set.long()
+                assert (torch.sort(to_set_long, descending=True)[0] == to_set_long).all()
         assert to_set.all(), (to_set, to_set.shape)
-    from interp.circuit.computational_node import make_index_at
 
     return [
         run_on_index(
@@ -1537,7 +1546,7 @@ def nested_einsum_axes_permute(c: Einsum, element_index: int, permutation: Seque
     new_out_axes = get_permuted(node.out_axes)
     new_node_axes = get_permuted(node_axes)
 
-    new_node = cast(Einsum, node.evolve(out_axes=new_out_axes).rename(node.name if keep_name else f"{node.name}_perm"))
+    new_node = Einsum(*node.args, out_axes=new_out_axes, name=node.op_name if keep_name else f"{node.name}_perm")
     result = reconstruct_by(c, element_index, new_node, new_node_axes, other)
     if result == c:
         raise NoopRewriteError("nested_einsum_axes_permute noop")
@@ -1643,7 +1652,6 @@ def insert_diag_mul_mask_many(
 # maybe use split_to_concat
 def split_to_concat_for_batch(circ: Circuit, batch_size: int, axis: int = 0):
     assert circ.rank >= 1
-    from interp.circuit.computational_node import make_index_at
 
     return Concat(
         *[
@@ -1652,3 +1660,49 @@ def split_to_concat_for_batch(circ: Circuit, batch_size: int, axis: int = 0):
         ],
         axis=axis,
     )
+
+
+# useful for pushing an index symbol "past" the symbol into module arguments
+def push_down_indexes_through_modules(
+    circ, matcher: MatcherIn = Matcher(True), push_down: Callable[[Index], Circuit] = push_down_index
+):
+    syms = set(get_free_symbols(circ))
+    no_parent_fvs = Matcher(lambda c: syms.isdisjoint(set(get_free_symbols(c))))
+
+    def is_sym(y):
+        return y.is_symbol() or (isinstance(y, SetSymbolicShape) and is_sym(y.children[0]))
+
+    def rename_idx_set_shape(y) -> Circuit:
+        if isinstance(y, SetSymbolicShape) or y.is_index():
+            y = y.map_child(0, rename_idx_set_shape)
+            return y.rename(y.get_autoname())
+        return y
+
+    def update_bind(y):
+        y = rename_idx_set_shape(y)
+        if y.is_index():
+            return push_down(y.cast_index())
+        return y
+
+    m = Matcher(Index) & (lambda y: is_sym(y.children[0])) & no_parent_fvs
+
+    def construct(m, old):
+        m2 = module_remove_unused_inputs(
+            m, use_elim_no_input_module=False, add_suffix_on_remove_unused=False
+        ).cast_module()
+        assert m2.num_children <= module_remove_unused_inputs(old).num_children, (
+            "push_down_indexes_through_modules would result in duplicating an input (index of it + it, or multiple different indexes of it)",
+            m2,
+        )
+        return m2
+
+    return pull_out_of_modules(
+        circ,
+        matcher,
+        update_new_binds=update_bind,
+        matcher_new_binds=m,
+        module_construct_callback=construct,
+        circuit_to_arg_spec=lambda c: ModuleArgSpec(
+            Symbol(c.shape, uuid.UUID(bytes=c.hash[:16]), c.get_unique(Symbol).name), True, True, True
+        ),
+    )
