@@ -1,10 +1,8 @@
 from typing import Callable, Tuple
 
-import numpy as np
 import torch
-from attr import frozen
 
-from rust_circuit import Array, DiscreteVar
+from rust_circuit import Array
 from rust_circuit.causal_scrubbing.dataset import Dataset
 
 
