@@ -1,5 +1,9 @@
 from time import perf_counter
-from typing import Generic, NoReturn, Optional, Tuple, TypeVar
+from typing import Generic, Iterable, List, Mapping, NoReturn, Optional, Tuple, TypeVar, Union
+
+import torch
+
+from ._rust import TorchAxisIndex
 
 
 def assert_never(x: NoReturn) -> NoReturn:
@@ -30,12 +34,29 @@ class timed:
             print(self.readout + (self.extra_print if self.extra_print is not None else ""))
 
 
+TorchIndex = Union[Tuple[TorchAxisIndex, ...], TorchAxisIndex]
+
+
 class Indexer:
-    def __getitem__(self, idx):
+    """Helper for defining slices more easily which always returns tuples
+    (instead of sometimes returning just TorchAxisIndex)."""
+
+    def __getitem__(self, idx: TorchIndex) -> Tuple[TorchAxisIndex, ...]:
+        if isinstance(idx, tuple):
+            return idx
+        return (idx,)
+
+
+class Slicer:
+    """Helper for defining slices more easily which always returns slices"""
+
+    def __getitem__(self, idx: slice) -> slice:
+        assert isinstance(idx, slice)
         return idx
 
 
 I = Indexer()
+S = Slicer()
 
 KT = TypeVar("KT")
 VT = TypeVar("VT")
@@ -68,3 +89,27 @@ class FrozenDict(dict, Generic[KT, VT]):
 
     def __repr__(self):
         return f"{self.__class__.__name__}({super().__repr__()})"
+
+
+def get_slice_list(n: int) -> List[TorchAxisIndex]:
+    return [slice(None)] * n
+
+
+def make_index_at(index: TorchIndex, at: int):
+    return make_index_at_many({at: index})
+
+
+def to_axis_index(idx: TorchIndex) -> TorchAxisIndex:
+    if isinstance(idx, tuple):
+        assert len(idx) == 1
+        return idx[0]
+    else:
+        return idx
+
+
+def make_index_at_many(at_idx: Mapping[int, TorchIndex]) -> Tuple[TorchAxisIndex, ...]:
+    out = get_slice_list(max(at_idx.keys(), default=-1) + 1)
+    for at, idx in at_idx.items():
+        out[at] = to_axis_index(idx)
+
+    return tuple(out)
