@@ -3,7 +3,6 @@ from typing import Callable, Optional
 import pytest
 import torch
 
-from interp.tools.indexer import TORCH_INDEXER as I
 from rust_circuit import (
     Add,
     Array,
@@ -21,7 +20,7 @@ from rust_circuit import (
 )
 from rust_circuit.causal_scrubbing.dataset import Dataset
 from rust_circuit.causal_scrubbing.experiment import Experiment, ExperimentEvalSettings
-from rust_circuit.causal_scrubbing.hypothesis import Correspondence, ExactSampler
+from rust_circuit.causal_scrubbing.hypothesis import Correspondence, ExactSampler, FixedOtherSampler
 from rust_circuit.causal_scrubbing.hypothesis import FuncSampler as FS
 from rust_circuit.causal_scrubbing.hypothesis import (
     IllegalCorrespondenceError,
@@ -30,6 +29,7 @@ from rust_circuit.causal_scrubbing.hypothesis import (
     corr_root_matcher,
 )
 from rust_circuit.causal_scrubbing.testing_utils import IntDataset, loss_fn
+from rust_circuit.py_utils import I
 
 label_cond = FS(lambda d: d.labels.value)
 
@@ -309,6 +309,7 @@ def test_corr_tree_structure_only_if():
 
 # === Unit tests ===
 
+
 # filter out warnings about check = False
 @pytest.mark.filterwarnings("ignore:You're")
 def test_treeify():
@@ -365,22 +366,28 @@ def test_set_inputs():
     C = Add(A, B, name="C")
     dataset = IntDataset.of_shape((5, 2), lambda x: 2 * x[0] + 3 * x[1])
 
+    # Create 2 definitely different datasets of the correct len
+    A_sample = dataset[0:1]
+    B_sample = IntDataset([Array(A_sample.xs.value + 1, "xs"), A_sample.labels])
+
     # Construct correspondence
     out = corr_root_matcher
     to_A = out.chain("A")
     corr = Correspondence()
-    C_t = InterpNode(name="C", cond_sampler=UncondSampler())
+    # other_inputs_sampler will be used for the input to B
+    C_t = InterpNode(name="C", cond_sampler=ExactSampler(), other_inputs_sampler=FixedOtherSampler(B_sample))
     corr.add(C_t, out)
-    corr.add(C_t.make_descendant(name="A", cond_sampler=UncondSampler()), to_A)
+    corr.add(C_t.make_descendant(name="A", cond_sampler=FixedOtherSampler(A_sample)), to_A)
 
     # First treeify, then set inputs
     ex = Experiment(C, dataset, corr, random_seed=1)
     ref_ds = ex.make_ref_ds(1)
     sampled_inputs = ex.sample(ref_ds)
+
     circuit = ex.treeified()
     assert num_input_tags(circuit) == 2
 
-    # Note this number depends on the sampler returning different datasets for C and A
+    # Note this number depends on the samplers returning different datasets for the treeified inputs; we've enforced this by construction
     circuit = ex.wrap_in_var(circuit, ref_ds, DiscreteVar.uniform_probs_and_group(1))
     circuit = ex.replace_inputs(circuit, sampled_inputs)
     assert len(IterativeMatcher("xs").get(circuit)) == 2
