@@ -81,6 +81,7 @@ from rust_circuit.rrfs import RRFS_DIR
 from .test_rust_rewrite import get_c_st
 
 P = Parser()
+po = rc.PrintOptions(tensor_index_literal=True, bijection=False)
 
 
 @hypothesis.given(get_c_st())
@@ -100,9 +101,9 @@ def test_push_down_modules(circ, d):
     skips = d.draw(st_c.st_subsets([x.name for x in all_children(circ) if x.is_module()]))
     cb = d.draw(st.builds(rc.ModulePusher.remove_and_elim_callback, st.booleans(), st.booleans(), st.booleans()))
     pu = rc.ModulePusher(flatten_modules=d.draw(st.booleans()), module_construct_callback=cb)
-    note(circ.repr())
+    note(circ.repr(po))
     circ1 = pu.push_down_modules(circ, traversal=rc.traversal, skip_module=skips)
-    note(circ1.repr())
+    note(circ1.repr(po))
     assert_close(circ.evaluate(), circ1.evaluate())
 
 
@@ -113,7 +114,7 @@ def test_push_down_modules(circ, d):
 )
 @mark_not_interesting_if(rc.ExtractSymbolsHasBindingsFromOuterModuleError)
 def test_extract_symbols(circ, d):
-    note(circ.repr())
+    note(circ.repr(po))
     syms = d.draw(st_c.st_subsets([x for x in all_children(circ) if x.is_symbol()]))
     conform = d.draw(st.booleans())
     try:
@@ -123,7 +124,7 @@ def test_extract_symbols(circ, d):
     except rc.ExtractSymbolsBatchedInputError:
         hypothesis.assume(conform)
         raise
-    note(circ1.repr())
+    note(circ1.repr(po))
     assert_close(circ.evaluate(), circ1.evaluate())
 
 
@@ -133,7 +134,7 @@ def test_extract_symbols(circ, d):
     st.data(),
 )
 def test_upd_nested_id(circ, d):
-    note(circ.repr())
+    note(circ.repr(po))
     try:
         circ1 = rc.update_bindings_nested(
             circ,
