@@ -296,7 +296,7 @@ def get_rewrite_st(setup):
     shape = st.sampled_from(options["shapes"]) if "shapes" in options else st_np.array_shapes(min_dims=1, max_dims=3)
     circuit_type_settings = {"Add": (1, add_settings)} | options.get("circuit_type_settings", {})
     probs_per_depth = [CP.kw(all=0, **{k: circuit_type_settings.get(k, 1) for k in x}) for x in probs_normalized]
-    return get_c_st(shape, probs_per_depth, max_growth_steps=20, from_other=False)
+    return get_c_st(shape, probs_per_depth, max_growth_steps=options.get("max_growth_steps", 20), from_other=False)
 
 
 def deep_pull_concat_strip(c: rCircuit) -> rCircuit:
@@ -400,7 +400,7 @@ rewrite_test_setups = [
     (concat_pull_removable_axes, ["Concat", ["Scalar", "Rearrange"]]),
     (einsum_pull_removable_axes, ["Einsum", ["Scalar", "Rearrange"]]),
     (add_make_broadcasts_explicit, ["Add"]),
-    (distribute_all, ["Einsum", "Add"]),
+    (distribute_all, ["Einsum", "Add"], {"max_growth_steps": 10}),
     (PartialName(distribute, operand_idx=0, do_broadcasts=True), ["Einsum", "Add"]),
     (einsum_of_permute_merge, ["Einsum", "Rearrange"]),  # i really don't know why this needs f64
     (permute_of_einsum_merge, ["Rearrange", "Einsum"]),
@@ -536,12 +536,13 @@ def raw_test_rewrite(
     do_print=False,
     trans: Optional[Circuit] = None,
 ):
-    note(circ.repr())
+    po = PrintOptions(tensor_index_literal=True, bijection=False)
+    note(circ.repr(po))
     # Don't canonicalize or strip names on purpose, we want circ to accurately specify what gets transformed.
     ddt = rc.TorchDeviceDtypeOp(dtype=dtype)
     circ = rc.cast_circuit(circ, ddt)
     if do_print:
-        PrintOptions(tensor_index_literal=True, bijection=False).print(circ)
+        circ.print(po)
     try:
         rewritten = rewrite(circ)
     except (
@@ -560,9 +561,9 @@ def raw_test_rewrite(
         assert rewritten is not None, "expected a transformation"
     else:
         hypothesis.assume(rewritten is not None, "Rewrite returned None (noop)")
-    note(rewritten.repr())
+    note(rewritten.repr(po))
     if do_print:
-        PrintOptions(tensor_index_literal=True, bijection=False).print(rewritten)
+        rewritten.print(po)
     if assert_different:
         assert rewritten != circ
     else:
