@@ -10,6 +10,8 @@ from rust_circuit import *
 
 from .test_rust_rewrite import get_c_st
 
+po = rc.PrintOptions(tensor_index_literal=True, bijection=False)
+
 
 @hypothesis.settings(deadline=None)
 @hypothesis.given(get_c_st(max_growth_steps=20))
@@ -20,7 +22,7 @@ def test_schedule(circ):
 
 def raw_test_schedule(circ):
     schedule = optimize_to_schedule(circ)
-    hypothesis.note(circ.repr())
+    hypothesis.note(circ.repr(po))
     # hypothesis.note(rc.optimize_circuit(circ).repr())
 
     hypothesis.assume(
